@@ -27,9 +27,10 @@ jest.mock('app/core/services/backend_srv', () => {
   };
 });
 
-jest.mock('@grafana/runtime/src/config', () => ({
+jest.mock('@grafana/runtime', () => ({
+  ...jest.requireActual('@grafana/runtime'),
   config: {
-    ...jest.requireActual('@grafana/runtime/src/config').config,
+    ...jest.requireActual('@grafana/runtime').config,
     buildInfo: {
       edition: 'Enterprise',
       version: '9.0.0',
